@@ -6,13 +6,12 @@ use 5.014;
 
 use Graphics::Grid::Class;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use List::AllUtils qw(reduce);
 use Math::Trig qw(:pi :radial deg2rad);
 use Module::Load;
 use Types::Standard qw(InstanceOf ConsumerOf ArrayRef HashRef Str);
-use namespace::autoclean;
 
 use Graphics::Grid::Viewport;
 use Graphics::Grid::ViewportTree;
@@ -34,11 +33,16 @@ has _current_vptree => (
 
 
 has driver => (
-    is      => 'rw',
-    lazy    => 1,
-    isa     => ConsumerOf ["Graphics::Grid::Driver"],
-    builder => '_build_driver',
-
+    is        => 'rw',
+    lazy      => 1,
+    isa       => ConsumerOf ["Graphics::Grid::Driver"],
+    builder   => '_build_driver',
+    predicate => 'has_driver',
+    trigger   => sub {
+        my ( $self, $driver ) = @_;
+        $driver->grid($self);
+        $self->_set_vptree();
+    },
 );
 
 has _gp_stack => (
@@ -52,25 +56,33 @@ has _gp_stack => (
     }
 );
 
-sub _build_driver {
+method _build_driver() {
     my $driver_cls = 'Graphics::Grid::Driver::Cairo';
     load $driver_cls;
-    return $driver_cls->new();
+    my $driver = $driver_cls->new( grid => $self );
+    $self->_set_vptree( $self->_current_vptree, $driver );
+    return $driver;
 }
 
 sub _build__vptree {
     my ($self) = @_;
-    return Graphics::Grid::ViewportTree->new(
+    my $vptree = Graphics::Grid::ViewportTree->new(
         node => Graphics::Grid::Viewport->new(
             name => 'ROOT',
-            gp   => $self->driver->default_gpar()
         )
     );
+    return $vptree;
 }
 
 sub _build__current_vptree { $_[0]->_vptree }
 
 
+classmethod singleton() {
+    state $instance = $class->new();
+    return $instance;
+}
+
+
 method current_vptree( $all = true ) {
     return ( $all ? $self->_vptree : $self->_current_vptree );
 }
@@ -208,9 +220,9 @@ method _seek_viewport( $from_tree, $name_or_path ) {
         if ( $self->_check_vppath( $tree, [@$path] ) ) {
             $self->_current_vptree($tree);
 
-            my $depth         = 0;
-            my $from_tree_uid = $from_tree->node->_uid;
-            while ( $tree->node->_uid ne $from_tree_uid ) {
+            my $depth          = 0;
+            my $from_tree_name = $from_tree->node->name;
+            while ( $tree->node->name ne $from_tree_name ) {
                 $depth++;
                 last unless ( $tree->has_parent );
                 $tree = $tree->parent;
@@ -244,6 +256,12 @@ method draw($grob) {
         $self->_set_vptree( $self->_vptree );
     }
 
+    # update grob/gtree pre-draw
+    $grob = $grob->make_context;
+    if ( $grob->can('make_content') ) {
+        $grob = $grob->make_content;
+    }
+
     if ( $grob->vp ) {
         $self->push_viewport( $grob->vp );
     }
@@ -252,17 +270,18 @@ method draw($grob) {
         $self->_push_gp( $grob->gp );
     }
     my @gp = reverse @{ $self->_gp_stack };
-    my $merged_gp = reduce { $a->merge($b) } $gp[0], @gp[ 1 .. $#gp ];
+    my $merged_gp = reduce { $a->merge($b) } $gp[0],
+      ( @gp[ 1 .. $#gp ], $self->driver->default_gpar );
     $self->driver->current_gp($merged_gp);
 
-    $grob->draw( $self->driver );
+    $grob->_draw($self);
 
     if ( $grob->gp ) {
         $self->_pop_gp();
     }
 
     if ( $grob->vp ) {
-        $self->pop_viewport( $grob->vp );
+        $self->pop_viewport();
     }
 }
 
@@ -289,20 +308,21 @@ for my $grob_type ( __PACKAGE__->_grob_types ) {
     *{$grob_type} = $func;
 }
 
-# set current viewport to state
-method _set_vptree( $vptree = $self->_current_vptree ) {
-    $self->driver->current_vptree($vptree);
-
-    my $path = $self->_current_vptree->path_from_root;
-    $self->_clear_gp;
-    $self->_push_gp( grep { defined $_ } map { $_->gp } @$path );
-}
-
 
 method write($filename) {
     $self->driver->write($filename);
 }
 
+# set current viewport to state
+method _set_vptree( $vptree = $self->_current_vptree, $driver = $self->driver )
+{
+    $driver->current_vptree($vptree);
+
+    my $path = $vptree->path_from_root;
+    $self->_clear_gp;
+    $self->_push_gp( grep { defined $_ } map { $_->gp } @$path );
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -319,7 +339,7 @@ Graphics::Grid - An incomplete port of the R "grid" library to Perl
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -396,8 +416,7 @@ name "grid".
 =back
 
 The target of this Perl Graphics::Grid library, as of today, is to have
-most of the R "grid"'s fundamental features mentioned above except for
-the grid-layout system. 
+most of the R "grid"'s fundamental features mentioned above.
 
 This Graphics::Grid module is the object interface of this libray. There is
 also a function interface L<Graphics::Grid::Functions>, which is more like
@@ -410,6 +429,10 @@ the interface of the R "grid" library.
 Set the device driver. The value needs to be a consumer of the L<Graphics::Grid::Driver>
 role. Default is a L<Graphics::Grid::Driver::Cairo> object.
 
+=head1 CLASS METHODS
+
+=head2 singleton()
+
 =head1 METHODS
 
 =head2 current_vptree($all=true)
@@ -561,7 +584,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
