@@ -4,16 +4,20 @@ package Graphics::Grid::Driver::Cairo;
 
 use Graphics::Grid::Class;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use Cairo;
 use Scalar::Util qw(looks_like_number);
-use List::AllUtils qw(min max pairwise);
+use List::AllUtils qw(minmax min max pairwise reduce);
 use Math::Trig qw(:pi :radial deg2rad);
 use Path::Tiny;
 use Types::Standard qw(Enum Str InstanceOf Num);
 
-use Graphics::Grid::Util qw(dots_to_cm cm_to_dots points_to_cm cm_to_points);
+use Graphics::Grid::Extents;
+use Graphics::Grid::Unit;
+use Graphics::Grid::Util qw(
+  dots_to_cm cm_to_dots points_to_cm cm_to_points
+);
 
 my $AntialiasMode = Enum [qw(default none gray subpixel)];
 my $Format =
@@ -44,7 +48,7 @@ has 'cairo' => (
 );
 
 
-has format => ( is => 'ro', isa => $Format, default => 'png' );
+has format => ( is => 'ro', isa => $Format, default => 'png', coerce => 1 );
 
 
 has 'surface' => (
@@ -69,28 +73,22 @@ has 'surface' => (
             croak('Your Cairo does not have PDF support!')
               unless Cairo::HAS_PDF_SURFACE;
             $surface = Cairo::PdfSurface->create_for_stream(
-                sub { $self->{DATA} .= $_[1] }, $self, $width, $height
-
-                  # $self->can('append_surface_data'), $self, $width, $height
-            );
+                sub { $self->{DATA} .= $_[1] },
+                $self, $width, $height );
         }
         elsif ( $self->format eq 'ps' ) {
             croak('Your Cairo does not have PostScript support!')
               unless Cairo::HAS_PS_SURFACE;
             $surface = Cairo::PsSurface->create_for_stream(
-                sub { $self->{DATA} .= $_[1] }, $self, $width, $height
-
-                  # $self->can('append_surface_data'), $self, $width, $height
-            );
+                sub { $self->{DATA} .= $_[1] },
+                $self, $width, $height );
         }
         elsif ( $self->format eq 'svg' ) {
             croak('Your Cairo does not have SVG support!')
               unless Cairo::HAS_SVG_SURFACE;
             $surface = Cairo::SvgSurface->create_for_stream(
-                sub { $self->{DATA} .= $_[1] }, $self, $width, $height
-
-                  # $self->can('append_surface_data'), $self, $width, $height
-            );
+                sub { $self->{DATA} .= $_[1] },
+                $self, $width, $height );
         }
         else {
             croak( "Unknown format '" . $self->format . "'" );
@@ -102,7 +100,7 @@ has 'surface' => (
 
 with qw(Graphics::Grid::Driver);
 
-method _set_vptree( $vptree, $old_vptree = undef ) {
+method _set_vptree ( $vptree, $old_vptree = undef ) {
     my $ctx = $self->cairo;
 
     my $path = $self->current_vptree->path_from_root;
@@ -110,28 +108,73 @@ method _set_vptree( $vptree, $old_vptree = undef ) {
     # reset to basic setup
     $self->_reset_transform();
 
-    # bypass root vp
-    shift @$path;
-
+    # initial vp width and height
     my $vp_width  = dots_to_cm( $self->width,  $self->dpi );
     my $vp_height = dots_to_cm( $self->height, $self->dpi );
 
-    for my $vp (@$path) {
-        my $x =
-          $self->_transform_width_to_cm( $vp->x, 0, $vp->gp, $vp_width,
-            $vp_height );
-        my $y =
-          $self->_transform_height_to_cm( $vp->y, 0, $vp->gp, $vp_width,
-            $vp_height );
-        my $width =
-          $self->_transform_width_to_cm( $vp->width, 0, $vp->gp, $vp_width,
-            $vp_height );
-        my $height =
-          $self->_transform_height_to_cm( $vp->height, 0, $vp->gp, $vp_width,
-            $vp_height );
+    my $unit1 = Graphics::Grid::Unit->new( 1, 'npc' );
+
+    # start from 1 to skip root vp
+    for my $i ( 1 .. $#$path ) {
+        my $vp = $path->[$i];
+
+        my ( $x, $y, $width, $height );
+
+        if ( $vp->layout_pos_col or $vp->layout_pos_row ) {
+            my $parent_vp = $path->[ $i - 1 ];
+            my $layout    = $parent_vp->layout;
+            die unless $layout;
+
+            my $layout_width  = $layout->width;
+            my $layout_height = $layout->height;
+
+            $width =
+              $self->_transform_width_to_cm(
+                $layout->cell_width( $vp->layout_pos_col ),
+                0, $vp->gp, $vp_width );
+            $height =
+              $self->_transform_height_to_cm(
+                $layout->cell_height( $vp->layout_pos_row ),
+                0, $vp->gp, $vp_height );
+
+            my $x_base = ( $unit1 - $layout->width ) * $layout->hjust;
+            my $y_base =
+              ( $unit1 - $layout->height ) * $layout->vjust + $layout->height;
+
+            $x = $self->_transform_width_to_cm(
+                (
+                    reduce { $a + $b } $x_base,
+                    map { $layout->widths->at($_) }
+                      ( 0 .. min( @{ $vp->layout_pos_col } ) - 1 )
+                ),
+                0,
+                $vp->gp
+              ) +
+              $width * 0.5;
+            $y = $self->_transform_height_to_cm(
+                (
+                    $y_base - reduce { $a + $b } undef,
+                    map { $layout->heights->at($_) }
+                      ( 0 .. min( @{ $vp->layout_pos_row } ) - 1 )
+                ),
+                0,
+                $vp->gp
+              ) -
+              $height * 0.5;
+        }
+        else {
+            $x = $self->_transform_width_to_cm( $vp->x, 0, $vp->gp, $vp_width );
+            $y =
+              $self->_transform_height_to_cm( $vp->y, 0, $vp->gp, $vp_height );
+            $width =
+              $self->_transform_width_to_cm( $vp->width, 0, $vp->gp,
+                $vp_width );
+            $height =
+              $self->_transform_height_to_cm( $vp->height, 0, $vp->gp,
+                $vp_height );
+        }
 
         $ctx->translate( $x, $y );
-
         if ( $vp->angle != 0 ) {
             $ctx->rotate( deg2rad( $vp->angle ) );
         }
@@ -146,15 +189,15 @@ method _set_vptree( $vptree, $old_vptree = undef ) {
     $self->_current_vp_height_cm($vp_height);
 }
 
-method _reset_transform() {
+has _matrix_cm_invert => ( is => 'rw' );
+
+method _reset_transform () {
     my $ctx = $self->cairo;
 
     # reset transfom
     my $identity_matrix = Cairo::Matrix->init_identity;
     $ctx->set_matrix($identity_matrix);
 
-    #$ctx->set_font_matrix($identity_matrix);
-
     # set basic transform below
 
     # Cairo's Y direction is inverse from that of the grid library
@@ -163,19 +206,54 @@ method _reset_transform() {
     # We defaultly use cm as unit
     $self->_set_scale_cm();
 
+    # Store this matrix
+    my $matrix_invert = $ctx->get_matrix;
+    Cairo::Matrix::invert($matrix_invert);
+    $self->_matrix_cm_invert($matrix_invert);
+
+    return;
 }
 
-method _set_scale_cm() {
+method _set_scale_cm () {
     my $ctx = $self->cairo;
     $ctx->scale( $self->dpi / 2.54, -$self->dpi / 2.54 );
 }
-method _unset_scale_cm() {
+
+method _unset_scale_cm () {
     my $ctx = $self->cairo;
     $ctx->scale( 2.54 / $self->dpi, -2.54 / $self->dpi );
 }
 
+method _transform_point_to_cm ($point) {
+    my $ctx = $self->cairo;
 
-method data() {
+    my $matrix        = $ctx->get_matrix();
+    my $matrix_invert = Cairo::Matrix::multiply( $self->_matrix_cm_invert,
+        Cairo::Matrix->init_identity );
+    my $matrix1 = Cairo::Matrix::multiply( $ctx->get_matrix, $matrix_invert );
+    my ( $x1, $y1 ) = $matrix1->transform_point(@$point);
+
+    return [ $x1, $y1 ];
+}
+
+# given a list of points [x, y], calculate the extents in the cm scale.
+method _calc_extents (@points) {
+    my @transformed = map { $self->_transform_point_to_cm($_) } @points;
+    my @x           = map { $_->[0] } @transformed;
+    my @y           = map { $_->[1] } @transformed;
+    my ( $xmin, $xmax ) = minmax(@x);
+    my ( $ymin, $ymax ) = minmax(@y);
+
+    return Graphics::Grid::Extents->new(
+        x      => Graphics::Grid::Unit->new($xmin, 'cm'),
+        y      => Graphics::Grid::Unit->new($ymin, 'cm'),
+        width  => Graphics::Grid::Unit->new($xmax - $xmin, 'cm'),
+        height => Graphics::Grid::Unit->new($ymax - $ymin, 'cm'),
+    );
+}
+
+
+method data () {
     my $ctx = $self->cairo;
 
     if ( $self->format eq 'png' ) {
@@ -198,7 +276,7 @@ method data() {
     return $self->{DATA};
 }
 
-method draw_circle($circle_grob) {
+method draw_circle ($circle_grob) {
     my $ctx = $self->cairo;
 
     my $vp_width  = $self->_current_vp_width_cm;
@@ -207,12 +285,8 @@ method draw_circle($circle_grob) {
     my $gp = $self->current_gp;
 
     for my $idx ( 0 .. $circle_grob->elems - 1 ) {
-        my $x =
-          $self->_transform_width_to_cm( $circle_grob->x, $idx, $gp,
-            $vp_width, $vp_height );
-        my $y =
-          $self->_transform_height_to_cm( $circle_grob->y, $idx, $gp,
-            $vp_width, $vp_height );
+        my $x = $self->_transform_width_to_cm( $circle_grob->x, $idx, $gp );
+        my $y = $self->_transform_height_to_cm( $circle_grob->y, $idx, $gp );
         my $r = $self->_transform_width_to_cm( $circle_grob->r, $idx, $gp,
             min( $vp_width, $vp_height ) );
 
@@ -228,7 +302,7 @@ method draw_circle($circle_grob) {
     }
 }
 
-method draw_rect($rect_grob) {
+method draw_rect ($rect_grob) {
     my $ctx = $self->cairo;
 
     my $vp_width  = $self->_current_vp_width_cm;
@@ -237,18 +311,12 @@ method draw_rect($rect_grob) {
     my $gp = $self->current_gp;
 
     for my $idx ( 0 .. $rect_grob->elems - 1 ) {
-        my $x =
-          $self->_transform_width_to_cm( $rect_grob->x, $idx, $gp, $vp_width,
-            $vp_height );
-        my $y =
-          $self->_transform_height_to_cm( $rect_grob->y, $idx, $gp, $vp_width,
-            $vp_height );
+        my $x = $self->_transform_width_to_cm( $rect_grob->x, $idx, $gp );
+        my $y = $self->_transform_height_to_cm( $rect_grob->y, $idx, $gp );
         my $width =
-          $self->_transform_width_to_cm( $rect_grob->width, $idx, $gp,
-            $vp_width, $vp_height );
+          $self->_transform_width_to_cm( $rect_grob->width, $idx, $gp );
         my $height =
-          $self->_transform_height_to_cm( $rect_grob->height, $idx, $gp,
-            $vp_width, $vp_height );
+          $self->_transform_height_to_cm( $rect_grob->height, $idx, $gp );
 
         my ( $left, $bottom ) =
           $rect_grob->calc_left_bottom( $x, $y, $width, $height );
@@ -274,7 +342,7 @@ my $path_func_lines = fun( $c, $points ) {
     }
 };
 
-method draw_segments($segments_grob) {
+method draw_segments ($segments_grob) {
     my $ctx = $self->cairo;
 
     my $gp = $self->current_gp;
@@ -302,11 +370,11 @@ method draw_segments($segments_grob) {
     }
 }
 
-method draw_polyline($polyline_grob) {
+method draw_polyline ($polyline_grob) {
     $self->_draw_polyline( $polyline_grob, $path_func_lines, false );
 }
 
-method draw_polygon($polygon_grob) {
+method draw_polygon ($polygon_grob) {
     $self->_draw_polyline(
         $polygon_grob,
         fun( $c, $points )
@@ -323,7 +391,7 @@ method draw_polygon($polygon_grob) {
     );
 }
 
-method _draw_polyline( $polyline_grob, $path_func, $is_fill = false ) {
+method _draw_polyline ( $polyline_grob, $path_func, $is_fill = false ) {
     my $ctx = $self->cairo;
 
     my $gp = $self->current_gp;
@@ -351,7 +419,7 @@ method _draw_polyline( $polyline_grob, $path_func, $is_fill = false ) {
     }
 }
 
-method _draw_shape( $gp, $path_func, $is_fill = false ) {
+method _draw_shape ( $gp, $path_func, $is_fill = false ) {
     my $ctx = $self->cairo;
 
     # draw path
@@ -424,21 +492,21 @@ method _draw_shape( $gp, $path_func, $is_fill = false ) {
     return 0;
 }
 
-method _set_color($gp) {
+method _set_color ($gp) {
     my $ctx   = $self->cairo;
     my $color = $gp->col->[0];
     $color->alpha( $gp->alpha->[0] ) if $gp->has_alpha;
     $ctx->set_source_rgba( $color->as_array_with_alpha );
 }
 
-method _set_fill($gp) {
+method _set_fill ($gp) {
     my $ctx   = $self->cairo;
     my $color = $gp->fill->[0];
     $color->alpha( $gp->alpha->[0] ) if defined $gp->has_alpha;
     $ctx->set_source_rgba( $color->as_array_with_alpha );
 }
 
-method _select_font_face($gp) {
+method _select_font_face ($gp) {
     my $ctx        = $self->cairo;
     my $fontfamily = $gp->fontfamily->[0];
     my $fontface   = $gp->fontface->[0];
@@ -455,7 +523,7 @@ method _select_font_face($gp) {
         @{ $fontface_to_params->{$fontface} } );
 }
 
-method draw_text($text_grob) {
+method draw_text ($text_grob, $query=false) {
     my $ctx = $self->cairo;
 
     my $vp_width  = $self->_current_vp_width_cm;
@@ -463,16 +531,14 @@ method draw_text($text_grob) {
 
     my $gp = $self->current_gp;
 
+    my @extents;
+
     for my $idx ( 0 .. $text_grob->elems - 1 ) {
         my $text = $text_grob->label->[$idx];
         next unless ( length($text) );
 
-        my $x =
-          $self->_transform_width_to_cm( $text_grob->x, $idx, $gp,
-            $vp_width, $vp_height );
-        my $y =
-          $self->_transform_height_to_cm( $text_grob->y, $idx, $gp,
-            $vp_width, $vp_height );
+        my $x = $self->_transform_width_to_cm( $text_grob->x, $idx, $gp );
+        my $y = $self->_transform_height_to_cm( $text_grob->y, $idx, $gp );
 
         my $gp_single = $gp->at($idx);
 
@@ -493,6 +559,8 @@ method draw_text($text_grob) {
 
         my ( $left, $bottom ) =
           $text_grob->calc_left_bottom( $x, $y, $width, $height );
+        $left   += $exts->{x_bearing};
+        $bottom += $exts->{y_bearing};
 
         $self->_set_color($gp_single);
 
@@ -506,16 +574,32 @@ method draw_text($text_grob) {
             $ctx->translate( -$x, -$y );
         }
         $ctx->move_to( $left, $bottom );
-        $ctx->show_text($text);
+
+        my $right = $left + $width;
+        my $top   = $bottom + $height;
+        push @extents,
+          $self->_calc_extents(
+            [ $left,  $bottom ],
+            [ $left,  $top ],
+            [ $right, $bottom ],
+            [ $right, $top ]
+          );
+
+        unless ($query) {
+            $ctx->show_text($text);
+        }
 
         $ctx->restore;
     }
+
+    my $extents = reduce { $a->merge($b) } @extents;
+    return $extents;
 }
 
 use Graphics::Grid::Grob::Text;
 
 # TODO: this is now just a poor man's implementation...
-method draw_points($points_grob) {
+method draw_points ($points_grob) {
     my $elems = $points_grob->elems;
     my $gp    = $self->current_gp;
 
@@ -554,7 +638,7 @@ method draw_points($points_grob) {
 }
 
 
-method write($file) {
+method write ($file) {
     path($file)->spew_raw( $self->data );
 }
 
@@ -574,7 +658,7 @@ Graphics::Grid::Driver::Cairo - Cairo backend for Graphics::Grid
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 DESCRIPTION
 
@@ -624,7 +708,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
