@@ -3,89 +3,116 @@ package Graphics::Grid::GPar;
 # ABSTRACT: Graphical parameters used in Graphics::Grid
 
 use Graphics::Grid::Class;
-use MooseX::HasDefaults::RO;
+use MooseX::Aliases;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
-use List::AllUtils;
+use List::AllUtils qw(pairgrep pairmap zip);
 use Graphics::Color::RGB;
 use Types::Standard qw(Num Enum ArrayRef Str Value Int);
 use Type::Utils qw(declare_coercion);
-use namespace::autoclean;
 
 use Graphics::Grid::Types qw(:all);
 
 my $LineMitre   = Num->where( sub { $_ >= 1 } );
 my $ZeroToOne   = Num->where( sub { $_ >= 0 and $_ <= 1 } );
 my $NonNegative = Num->where( sub { $_ >= 0 } );
+my $Color = ( ArrayRef [Color] )->plus_coercions( Color, sub { [$_] } )
+  ->plus_coercions( Str, sub { [ Color->coerce($_) ] } );
+
+around BUILDARGS( $orig, $class : @rest ) {
+    my %params = @rest == 1 ? %{ $rest[0] } : @rest;
+    return $class->$orig( pairgrep { defined $b } %params );
+}
 
 # color properties
-has [qw(col fill)] => (
-    isa => (
-        ( ArrayRef [Color] )->plus_coercions( Color, sub { [$_] } )
-          ->plus_coercions( Str, sub { [ Color->coerce($_) ] } )
-    ),
+has col => (
+    is     => 'ro',
+    isa    => $Color,
+    coerce => 1,
+    alias  => [qw(color colour)],
+);
+has fill => (
+    is     => 'ro',
+    isa    => $Color,
     coerce => 1,
 );
 has alpha => (
-    isa     => ( ArrayRef [$ZeroToOne] )->plus_coercions(ArrayRefFromValue),
+    is      => 'ro',
+    isa     => ( ArrayRef [$ZeroToOne] )->plus_coercions(ArrayRefFromAny),
     coerce  => 1,
     default => sub { [1] },
 );
 
 # line properties
 has lty => (
-    isa => ( ArrayRef [LineType] )->plus_coercions(ArrayRefFromValue),
-    coerce => 1
+    is     => 'ro',
+    isa    => ( ArrayRef [LineType] )->plus_coercions(ArrayRefFromAny),
+    coerce => 1,
+    alias  => 'linetype',
 );
 has lwd => (
-    isa => ( ArrayRef [$NonNegative] )->plus_coercions(ArrayRefFromValue),
-    coerce => 1
+    is     => 'ro',
+    isa    => ( ArrayRef [$NonNegative] )->plus_coercions(ArrayRefFromAny),
+    coerce => 1,
+    alias  => 'linewidth',
 );
 has lex => (
-    isa     => ( ArrayRef [$NonNegative] )->plus_coercions(ArrayRefFromValue),
+    is      => 'ro',
+    isa     => ( ArrayRef [$NonNegative] )->plus_coercions(ArrayRefFromAny),
     coerce  => 1,
     default => sub { [1] },
 );
 has lineend => (
-    isa => ( ArrayRef [LineEnd] )->plus_coercions(ArrayRefFromValue),
+    is     => 'ro',
+    isa    => ( ArrayRef [LineEnd] )->plus_coercions(ArrayRefFromAny),
     coerce => 1
 );
 has linejoin => (
-    isa => ( ArrayRef [LineJoin] )->plus_coercions(ArrayRefFromValue),
+    is     => 'ro',
+    isa    => ( ArrayRef [LineJoin] )->plus_coercions(ArrayRefFromAny),
     coerce => 1
 );
 has linemitre => (
-    isa => ( ArrayRef [$LineMitre] )->plus_coercions(ArrayRefFromValue),
+    is     => 'ro',
+    isa    => ( ArrayRef [$LineMitre] )->plus_coercions(ArrayRefFromAny),
     coerce => 1
 );
 
 # text properties
 has fontsize => (
-    isa => ( ArrayRef [$NonNegative] )->plus_coercions(ArrayRefFromValue),
+    is     => 'ro',
+    isa    => ( ArrayRef [$NonNegative] )->plus_coercions(ArrayRefFromAny),
     coerce => 1
 );
 has fontfamily => (
-    isa => ( ArrayRef [Str] )->plus_coercions(ArrayRefFromValue),
-    coerce => 1
+    is     => 'ro',
+    isa    => ( ArrayRef [Str] )->plus_coercions(ArrayRefFromAny),
+    coerce => 1,
+    alias  => 'family',
 );
 has fontface => (
-    isa => ( ArrayRef [FontFace] )->plus_coercions(ArrayRefFromValue),
-    coerce => 1
+    is     => 'ro',
+    isa    => ( ArrayRef [FontFace] )->plus_coercions(ArrayRefFromAny),
+    coerce => 1,
+    alias  => 'face',
 );
 has lineheight => (
-    isa    => (ArrayRef)->plus_coercions(ArrayRefFromValue),
+    is     => 'ro',
+    isa    => ( ArrayRef [$NonNegative] )->plus_coercions(ArrayRefFromAny),
     coerce => 1
 );
 
 # other properties
 has cex => (
-    isa     => ( ArrayRef [$NonNegative] )->plus_coercions(ArrayRefFromValue),
+    is      => 'ro',
+    isa     => ( ArrayRef [$NonNegative] )->plus_coercions(ArrayRefFromAny),
     coerce  => 1,
     default => sub { [1] },
 );
 
-classmethod names() {
+
+classmethod names () {
     my @names = qw(col fill alpha
       lty lwd lex lineend linejoin linemitre
       fontsize fontfamily fontface lineheight cex
@@ -93,7 +120,7 @@ classmethod names() {
     return \@names;
 }
 
-method _has_param($name) {
+method _has_param ($name) {
     my $val = $self->$name;
     return ( defined $val and @{$val} > 0 );
 }
@@ -104,7 +131,7 @@ for my $name ( @{ __PACKAGE__->names } ) {
 }
 
 
-method at($idx) {
+method at ($idx) {
     my %params = map {
         my $val =
             $self->_has_param($_)
@@ -116,7 +143,7 @@ method at($idx) {
 }
 
 
-method merge($another_gpar) {
+method merge ($another_gpar) {
     my %cumulative_names = map { $_ => 1 } qw(alpha lex cex);
 
     my %params = map {
@@ -126,8 +153,8 @@ method merge($another_gpar) {
             my @self_val    = @{ $self->$key };
             my @another_val = @{ $another_gpar->$key };
             $val = [
-                List::AllUtils::pairmap { $a * $b }
-                map { $_ // 1 } List::AllUtils::zip( @self_val, @another_val )
+                pairmap { $a * $b }
+                map { $_ // 1 } zip( @self_val, @another_val )
             ];
         }
         else {
@@ -158,7 +185,7 @@ Graphics::Grid::GPar - Graphical parameters used in Graphics::Grid
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -299,6 +326,10 @@ Also note that dependending on the device driver implementation, some of the
 parameters may not be effective. For example, C<lineheight> is not effective
 with Graphics::Grid::Driver::Cairo.
 
+=head1 CLASS METHODS
+
+=head2 names()
+
 =head1 METHODS
 
 =head2 at($idx)
@@ -335,7 +366,7 @@ would not have this parameter either. For example,
     Graphics::Grid::GPar->new(col => 'red', fill => 'blue');
 
 For a cumulative parameter, the values of $gp1 and $gp2 are multiplied. If
-the arrayrefs in $gp1 and $gp2 are not of same length, the shorter one
+the array refs in $gp1 and $gp2 are not of same length, the shorter one
 would be padded with 1 when multiplying. That is,
 
     my $gp1 = Graphics::Grid::GPar->new(lex => [1, 2, 3]);
@@ -399,7 +430,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
