@@ -4,45 +4,49 @@ package Graphics::Grid::GTree;
 
 use Graphics::Grid::Class;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 extends qw(Forest::Tree);
 
 use Types::Standard qw(ArrayRef InstanceOf);
-use namespace::autoclean;
 
 has '+children' => ( isa => ArrayRef [ InstanceOf ['Graphics::Grid::GTree'] ] );
 
-with qw(Graphics::Grid::Grob);
+with qw(
+  MooseX::Clone
+  Graphics::Grid::Grob
+);
 
-around BUILDARGS => sub {
-    my $orig  = shift;
-    my $class = shift;
-
-    my %params = @_;
+around BUILDARGS($orig, $class : @rest) {
+    my %params = @rest;
     my $children = ( delete $params{children} ) // [];
     $children =
       [ map { $_->$_isa(__PACKAGE__) ? $_ : __PACKAGE__->new( node => $_ ); }
           @$children ];
 
     $class->$orig( %params, children => $children );
-};
+}
 
 method _build_elems() {
     return $self->child_count;
 }
 
-method draw($driver) {
+method _draw($grid) {
     for my $child ( @{ $self->children } ) {
         if ( $child->node->$_can('draw') ) {
-            $child->node->draw($driver);
+            $child->node->draw($grid);
         }
         else {
-            $child->draw($driver);
+            $child->draw($grid);
         }
     }
 }
 
+
+method make_content() {
+    return $self->clone;
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -59,7 +63,7 @@ Graphics::Grid::GTree - gtree
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -84,6 +88,12 @@ all of its children.
 
 It is a subclass of L<Forest::Tree>.
 
+=head1 METHODS
+
+=head2 make_content()
+
+A hook to allow a derived class to modify its children before being drawn.
+
 =head1 SEE ALSO
 
 L<Graphics::Grid::Grob>
@@ -96,7 +106,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
