@@ -4,7 +4,7 @@ package Graphics::Grid::Grob::Circle;
 
 use Graphics::Grid::Class;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use List::AllUtils qw(max);
 
@@ -24,14 +24,21 @@ with qw(
   Graphics::Grid::Positional
 );
 
-method _build_elems() {
+method _build_elems () {
     return max( map { $self->$_->elems } qw(x y r) );
 }
 
-method draw($driver) {
-    $driver->draw_circle($self);
+method _draw ($grid) {
+    return $grid->driver->draw_circle($self);
 }
 
+#method extents ($grid) {
+#    my @v =
+#      map { $grid->driver->_transform_width_to_cm( $self->r, $_ ); }
+#      ( 0 .. $self->elems - 1 );
+#    return Graphics::Grid::Unit->new( \@v, 'cm' );
+#}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -48,7 +55,7 @@ Graphics::Grid::Grob::Circle - Circle grob
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -102,6 +109,19 @@ Get number of sub-elements in the grob.
 Grob classes shall implement a C<_build_elems()> method to support this
 attribute.
 
+=head1 METHODS
+
+=head2 length
+
+This is an alias of C<elems>.
+
+=head2 extents($grid)
+
+Returns info about the grob's extents (bounding box, etc) on the
+drawing layer, in cm.
+
+Note that not all grob classes have got this method implemented.
+
 For this module C<elems> returns the number of circles.
 
 =head1 SEE ALSO
@@ -116,7 +136,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
