@@ -3,19 +3,23 @@ package Graphics::Grid::Grob::Points;
 # ABSTRACT: Points grob
 
 use Graphics::Grid::Class;
-use MooseX::HasDefaults::RO;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use Graphics::Grid::Unit;
 use Graphics::Grid::Types qw(:all);
 
 
 
-has pch => ( isa => PlottingCharacter, default => 1 );
+has pch => (
+    is      => 'ro',
+    isa     => PlottingCharacter,
+    default => 1
+);
 
 
 has size => (
+    is      => 'ro',
     isa     => Unit,
     coerce  => 1,
     default => sub { Graphics::Grid::Unit->new( 1, "char" ) },
@@ -38,11 +42,11 @@ has '+y' => (
     }
 );
 
-method _build_elems() {
+method _build_elems () {
     return $self->x->elems;
 }
 
-method validate() {
+method validate () {
     unless (
         List::AllUtils::all { $self->$_->isa('Graphics::Grid::Unit') }
         qw(x y size)
@@ -58,8 +62,8 @@ method validate() {
     }
 }
 
-method draw($driver) {
-    $driver->draw_points($self);
+method _draw ($grid) {
+    $grid->driver->draw_points($self);
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -78,7 +82,7 @@ Graphics::Grid::Grob::Points - Points grob
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -140,6 +144,19 @@ Get number of sub-elements in the grob.
 Grob classes shall implement a C<_build_elems()> method to support this
 attribute.
 
+=head1 METHODS
+
+=head2 length
+
+This is an alias of C<elems>.
+
+=head2 extents($grid)
+
+Returns info about the grob's extents (bounding box, etc) on the
+drawing layer, in cm.
+
+Note that not all grob classes have got this method implemented.
+
 For this module C<elems> returns the number of points.
 
 =head1 SEE ALSO
@@ -154,7 +171,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
