@@ -3,9 +3,8 @@ package Graphics::Grid::Grob::Polygon;
 # ABSTRACT: Polygon grob
 
 use Graphics::Grid::Class;
-use MooseX::HasDefaults::RO;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 extends qw(Graphics::Grid::Grob::Polyline);
 
@@ -23,8 +22,8 @@ has '+x' =>
 has '+y' =>
   ( default => sub { Graphics::Grid::Unit->new( [ 0.5, 1, 0.5, 0 ] ) } );
 
-method draw($driver) {
-    $driver->draw_polygon($self);
+method _draw($grid) {
+    $grid->driver->draw_polygon($self);
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -43,7 +42,7 @@ Graphics::Grid::Grob::Polygon - Polygon grob
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -105,7 +104,7 @@ line.
 
 =head2 gp
 
-An object of Graphics::Grid::GPar. Default is an empty gpar object.
+An object of L<Graphics::Grid::GPar>. Default is an empty gpar object.
 
 =head2 vp
 
@@ -122,6 +121,19 @@ Get number of sub-elements in the grob.
 Grob classes shall implement a C<_build_elems()> method to support this
 attribute.
 
+=head1 METHODS
+
+=head2 length
+
+This is an alias of C<elems>.
+
+=head2 extents($grid)
+
+Returns info about the grob's extents (bounding box, etc) on the
+drawing layer, in cm.
+
+Note that not all grob classes have got this method implemented.
+
 For this module C<elems> returns the number of polygons. 
 
 =head1 SEE ALSO
@@ -138,7 +150,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
