@@ -3,9 +3,8 @@ package Graphics::Grid::Grob::Polyline;
 # ABSTRACT: Polyline grob
 
 use Graphics::Grid::Class;
-use MooseX::HasDefaults::RO;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use List::AllUtils qw(uniq);
 use Types::Standard qw(ArrayRef Int);
@@ -14,7 +13,10 @@ use Graphics::Grid::Unit;
 use Graphics::Grid::Types qw(:all);
 
 
-has id => ( isa => ArrayRef [Int] );
+has id => (
+    is  => 'ro',
+    isa => ArrayRef [Int]
+);
 
 # TODO
 # has arrow => ( isa => ArrayRef[$Arrow] );
@@ -44,11 +46,11 @@ has [qw(+x +y)] => (
 );
 
 
-method _build_elems() {
+method _build_elems () {
     return scalar( @{ $self->_ids } );
 }
 
-method _build__ids() {
+method _build__ids () {
     if ( !$self->has_id ) {
         return [0];
     }
@@ -58,7 +60,7 @@ method _build__ids() {
     }
 }
 
-method _build__indexes_by_id() {
+method _build__indexes_by_id () {
     if ( !$self->has_id ) {
         return { 0 => [ 0 .. $self->x->elems - 1 ] };
     }
@@ -73,21 +75,21 @@ method _build__indexes_by_id() {
 }
 
 
-method indexes_by_id($id) {
+method indexes_by_id ($id) {
     return $self->_indexes_by_id->{$id};
 }
 
 
-method unique_ids() {
+method unique_ids () {
     return $self->_ids;
 }
 
-method get_idx_by_id($id) {
+method get_idx_by_id ($id) {
     my @indexes = map { $_ == $id } @{ $self->id };
     return \@indexes;
 }
 
-method _has_param($name) {
+method _has_param ($name) {
     my $val = $self->$name;
     return ( defined $val and @{$val} > 0 );
 }
@@ -97,7 +99,7 @@ for my $name (qw(id arrow)) {
     *{ "has_" . $name } = sub { $_[0]->_has_param($name); }
 }
 
-method validate() {
+method validate () {
     my $x_size = $self->x->elems;
     my $y_size = $self->y->elems;
     unless ( $x_size == $y_size ) {
@@ -108,8 +110,8 @@ method validate() {
     }
 }
 
-method draw($driver) {
-    $driver->draw_polyline($self);
+method _draw ($grid) {
+    $grid->driver->draw_polyline($self);
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -128,7 +130,7 @@ Graphics::Grid::Grob::Polyline - Polyline grob
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -193,7 +195,7 @@ line.
 
 =head2 gp
 
-An object of Graphics::Grid::GPar. Default is an empty gpar object.
+An object of L<Graphics::Grid::GPar>. Default is an empty gpar object.
 
 =head2 vp
 
@@ -210,11 +212,22 @@ Get number of sub-elements in the grob.
 Grob classes shall implement a C<_build_elems()> method to support this
 attribute.
 
+=head1 METHODS
+
+=head2 length
+
+This is an alias of C<elems>.
+
+=head2 extents($grid)
+
+Returns info about the grob's extents (bounding box, etc) on the
+drawing layer, in cm.
+
+Note that not all grob classes have got this method implemented.
+
 For this module C<elems> returns the number of lines (number of unique
 C<id>) of a object.
 
-=head1 METHODS
-
 =head2 indexes_by_id($id)
 
 Get unit indexes of attributes C<x>, C<y>, C<id>, for a given id.
@@ -239,7 +252,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
