@@ -3,9 +3,8 @@ package Graphics::Grid::Grob::Rect;
 # ABSTRACT: Rectangular grob
 
 use Graphics::Grid::Class;
-use MooseX::HasDefaults::RO;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use List::AllUtils qw(max);
 
@@ -14,17 +13,23 @@ with qw(
   Graphics::Grid::Grob
   Graphics::Grid::Positional
   Graphics::Grid::Dimensional
-  Graphics::Grid::Justifiable
+  Graphics::Grid::HasJust
 );
 
-method _build_elems() {
+method _build_elems () {
     return max( map { $self->$_->elems } qw(x y width height) );
 }
 
-method draw($driver) {
-    $driver->draw_rect($self);
+method _draw ($grid) {
+    return $grid->driver->draw_rect($self);
 }
 
+#method extents ($grid) {
+#    my @v = map { $grid->driver->_transform_width_to_cm( $self->width, $_ ) }
+#      ( 0 .. $self->elems - 1 );
+#    return Graphics::Grid::Unit->new( \@v, 'cm' );
+#}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -41,7 +46,7 @@ Graphics::Grid::Grob::Rect - Rectangular grob
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -131,7 +136,7 @@ A reader accessor for vertical justification.
 
 =head2 gp
 
-An object of Graphics::Grid::GPar. Default is an empty gpar object.
+An object of L<Graphics::Grid::GPar>. Default is an empty gpar object.
 
 =head2 vp
 
@@ -148,6 +153,19 @@ Get number of sub-elements in the grob.
 Grob classes shall implement a C<_build_elems()> method to support this
 attribute.
 
+=head1 METHODS
+
+=head2 length
+
+This is an alias of C<elems>.
+
+=head2 extents($grid)
+
+Returns info about the grob's extents (bounding box, etc) on the
+drawing layer, in cm.
+
+Note that not all grob classes have got this method implemented.
+
 For this module C<elems> returns the number of rectangles.
 
 =head1 SEE ALSO
@@ -162,7 +180,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
