@@ -3,9 +3,8 @@ package Graphics::Grid::Grob::Segments;
 # ABSTRACT: Line segments grob
 
 use Graphics::Grid::Class;
-use MooseX::HasDefaults::RO;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use Types::Standard qw(ArrayRef Int);
 
@@ -37,8 +36,8 @@ method _build_elems() {
     return List::AllUtils::max( map { $self->$_->elems } qw(x0 y0 x1 y1) );
 }
 
-method draw($driver) {
-    $driver->draw_segments($self);
+method _draw($grid) {
+    $grid->driver->draw_segments($self);
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -57,7 +56,7 @@ Graphics::Grid::Grob::Segments - Line segments grob
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -99,7 +98,7 @@ A Graphics::Grid::Unit object specifying the stopping y-values of the line segme
 
 =head2 gp
 
-An object of Graphics::Grid::GPar. Default is an empty gpar object.
+An object of L<Graphics::Grid::GPar>. Default is an empty gpar object.
 
 =head2 vp
 
@@ -116,6 +115,19 @@ Get number of sub-elements in the grob.
 Grob classes shall implement a C<_build_elems()> method to support this
 attribute.
 
+=head1 METHODS
+
+=head2 length
+
+This is an alias of C<elems>.
+
+=head2 extents($grid)
+
+Returns info about the grob's extents (bounding box, etc) on the
+drawing layer, in cm.
+
+Note that not all grob classes have got this method implemented.
+
 For this module C<elems> always returns 1.
 
 =head1 SEE ALSO
@@ -132,7 +144,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
