@@ -3,9 +3,8 @@ package Graphics::Grid::Grob::Text;
 # ABSTRACT: Text grob
 
 use Graphics::Grid::Class;
-use MooseX::HasDefaults::RO;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use Types::Standard qw(Str ArrayRef Bool Num);
 
@@ -14,35 +13,43 @@ use Graphics::Grid::Unit;
 
 
 has label => (
-    isa      => ( ArrayRef [Str] )->plus_coercions(ArrayRefFromValue),
+    is => 'ro',
+    isa      => ( ArrayRef [Str] )->plus_coercions(ArrayRefFromAny),
     coerce   => 1,
     required => 1,
 );
 
 
 has rot => (
-    isa => ( ArrayRef [Num] )->plus_coercions(ArrayRefFromValue),
+    is => 'ro',
+    isa => ( ArrayRef [Num] )->plus_coercions(ArrayRefFromAny),
     coerce => 1,
     default => sub { [0] },
 );
 
-#has check_overlap => ( is => 'ro', isa => Bool, default => 0 );
+#has check_overlap => ( is => 'ro', default => 0 );
 
 
 with qw(
   Graphics::Grid::Grob
   Graphics::Grid::Positional
-  Graphics::Grid::Justifiable
+  Graphics::Grid::HasJust
 );
 
 method _build_elems() {
     return scalar( @{ $self->label } );
 }
 
-method draw($driver) {
-    $driver->draw_text($self);
+method _draw($grid) {
+    return $grid->driver->draw_text($self);
 }
 
+method extents($grid) {
+    return $grid->driver->draw_text($self, true);
+}
+
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -57,7 +64,7 @@ Graphics::Grid::Grob::Text - Text grob
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -146,7 +153,7 @@ The angle to rotate the text.
 
 =head2 gp
 
-An object of Graphics::Grid::GPar. Default is an empty gpar object.
+An object of L<Graphics::Grid::GPar>. Default is an empty gpar object.
 
 =head2 vp
 
@@ -163,6 +170,19 @@ Get number of sub-elements in the grob.
 Grob classes shall implement a C<_build_elems()> method to support this
 attribute.
 
+=head1 METHODS
+
+=head2 length
+
+This is an alias of C<elems>.
+
+=head2 extents($grid)
+
+Returns info about the grob's extents (bounding box, etc) on the
+drawing layer, in cm.
+
+Note that not all grob classes have got this method implemented.
+
 For this module C<elems> returns the number of texts in C<label>.
 
 =head1 SEE ALSO
@@ -175,7 +195,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
