@@ -4,15 +4,28 @@ package Graphics::Grid::Grob::Zero;
 
 use Graphics::Grid::Class;
 
+our $VERSION = '0.001'; # VERSION
+
+use Graphics::Grid::Extents;
+use Graphics::Grid::Unit;
+
 with qw(
   Graphics::Grid::Grob
 );
 
-our $VERSION = '0.0001'; # VERSION
+method _build_elems () { 0 }
 
-method _build_elems() { 0 }
+method _draw ($grid) { }
 
-method draw($driver) { }
+method extents ($grid) {
+    my $zero = Graphics::Grid::Unit->new(0, 'cm');
+    return Graphics::Grid::Extents->new(
+        x      => $zero,
+        y      => $zero,
+        width  => $zero,
+        height => $zero,
+    );
+}
 
 1;
 
@@ -28,7 +41,7 @@ Graphics::Grid::Grob::Zero - Empty grob with minimal size
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -53,7 +66,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
