@@ -5,7 +5,7 @@ package Graphics::Grid::Setup;
 use strict;
 use warnings;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use utf8;
 use feature ':5.14';
@@ -15,11 +15,15 @@ use Import::Into;
 use Carp;
 use Data::Dumper ();
 use Function::Parameters 2.0;
-use Safe::Isa                 ();
-use boolean                   ();
-use Moose                     ();
+use PerlX::Maybe ();
+use Safe::Isa    ();
+use boolean      ();
+
+use Moose 2.1400;
 use Moose::Role               ();
+use MooseX::LazyRequire       ();
 use MooseX::StrictConstructor ();
+use namespace::autoclean 0.20;
 
 use List::AllUtils qw(uniq);
 
@@ -52,23 +56,30 @@ sub _import_tag {
         Carp->import::into($target);
         Data::Dumper->import::into($target);
         Function::Parameters->import::into($target);
+        PerlX::Maybe->import::into($target);
         Safe::Isa->import::into($target);
         boolean->import::into($target);
     }
     elsif ( $tag eq ':class' ) {
         $class->_import_tag( $target, ':base' );
 
-        Function::Parameters->import::into( $target, qw(classmethod) );
+        Function::Parameters->import::into( $target,
+            qw(classmethod :modifiers) );
 
         Moose->import::into($target);
+        MooseX::LazyRequire->import::into($target);
         MooseX::StrictConstructor->import::into($target);
+        namespace::autoclean->import::into($target);
     }
     elsif ( $tag eq ':role' ) {
         $class->_import_tag( $target, ':base' );
 
-        Function::Parameters->import::into( $target, qw(classmethod) );
+        Function::Parameters->import::into( $target,
+            qw(classmethod :modifiers) );
 
         Moose::Role->import::into($target);
+        MooseX::LazyRequire->import::into($target);
+        namespace::autoclean->import::into($target);
     }
     else {
         croak qq["$tag" is not exported by the $class module\n];
@@ -89,7 +100,7 @@ Graphics::Grid::Setup - Import stuffs into Graphics::Grid classes
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -111,7 +122,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
