@@ -4,24 +4,16 @@ package Graphics::Grid::UnitArithmetic;
 
 use Graphics::Grid::Class;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 use Scalar::Util qw(looks_like_number);
 use Type::Params ();
-use Types::Standard qw(Str ArrayRef Any Num);
-use namespace::autoclean;
+use Types::Standard qw(Str ArrayRef Any Num Maybe);
 
-use Graphics::Grid::Util qw(points_to_cm);
 use Graphics::Grid::Types qw(:all);
 
 extends 'Forest::Tree';
 
-use overload
-  "+"        => 'plus',
-  "-"        => 'minus',
-  "*"        => 'multiply',
-  "fallback" => 1;
-
 
 has '+node' => (
     isa => (
@@ -52,39 +44,105 @@ has '+children' => (
     coerce => 1,
 );
 
+
+method elems() {
+    if ( $self->is_unit ) {
+        return $self->node->elems;
+    }
+    elsif ( $self->is_arithmetic ) {
+        return List::AllUtils::max( map { $_->elems } @{ $self->children } );
+    }
+    else {
+        return scalar( @{ $self->node } );
+    }
+}
+
+method is_null_unit() {
+    if ( $self->is_unit ) {
+        return $self->node->is_null_unit;
+    }
+    elsif ( $self->is_arithmetic ) {
+        return List::AllUtils::all { $_->is_null_unit } @{ $self->children };
+    }
+    else {
+        return true;
+    }
+}
+
 with qw(
   Graphics::Grid::UnitLike
 );
 
 
 method at($idx) {
+    return $self->slice( [$idx] );
+}
+
+method slice($indices) {
+    my $class = ref($self);
     if ( $self->is_unit ) {
-        return __PACKAGE__->new( node => $self->node->at($idx) );
+        return $class->new( node => $self->node->slice($indices) );
     }
     elsif ( $self->is_number ) {
-        return __PACKAGE__->new(
-            node => [ $self->node->[ $idx % $self->elems ] ] );
+        return $class->new(
+            node => [ @{ $self->node }[ map { $_ % $self->elems } @$indices ] ]
+        );
     }
     else {
-        return __PACKAGE__->new(
-            node     => $self->node,
-            children => [ map { $_->at($idx) } @{ $self->children } ]
+        my @new_indices = map { $_ % $self->elems } @$indices;
+        return $class->new(
+            node => $self->node,
+            children =>
+              [ map { $_->slice( \@new_indices ) } @{ $self->children } ]
         );
     }
 }
 
 
-method elems() {
+method string() {
     if ( $self->is_unit ) {
-        return $self->node->elems;
+        return $self->node->string;
     }
     elsif ( $self->is_number ) {
-        return scalar( @{ $self->node } );
+        return join( ', ', @{ $self->node } );
     }
     else {
-        return List::AllUtils::max( map { $_->elems } @{ $self->children } );
+        return join(
+            ', ',
+            map {
+                my $left  = $self->children->[0]->at($_);
+                my $right = $self->children->[1]->at($_);
+                my $format;
+                if ( $self->node eq '*' ) {
+
+                    # Swap number to left, to be in accordance with R 'grid'
+                    #  library.
+                    if ( $right->is_number ) {
+                        ( $left, $right ) = ( $right, $left );
+                    }
+                    if ( $right->is_arithmetic and $right->node ne '*' ) {
+                        $format = "%s%s(%s)";
+                    }
+                }
+                $format //= "%s%s%s";
+                sprintf( $format, $left->string, $self->node, $right->string );
+            } ( 0 .. $self->elems - 1 )
+        );
     }
-};
+}
+
+method _make_operation( $op, $other, $swap = undef ) {
+    if ( $other->$_isa('Graphics::Grid::UnitList') ) {
+        require Graphics::Grid::UnitList;
+        return $other->_make_operation( $op, $self, !$swap );
+    }
+
+    my $class = ref($self);
+    return $class->new(
+        node     => $op,
+        children => ( $swap ? [ $other, $self ] : [ $self, $other ] )
+    );
+}
 
 
 method is_unit() {
@@ -101,56 +159,89 @@ method is_arithmetic() {
     return !( $self->is_unit() or $self->is_number() );
 }
 
-
-method stringify() {
+method transform_to_cm( $grid, $idx, $gp, $length_cm ) {
+    my $driver = $grid->driver;
     if ( $self->is_unit ) {
-        return $self->node->stringify;
+        return $self->node->transform_to_cm( $grid, $idx, $gp, $length_cm );
     }
     elsif ( $self->is_number ) {
-        return join( ', ', @{ $self->node } );
+        return $self->node->[ $idx % scalar( @{ $self->node } ) ];
+    }
+    else {    # is_arithmetic
+        my $op = $self->node;
+        my ( $left, $right ) =
+          map { $_->transform_to_cm( $grid, $idx, $gp, $length_cm ) }
+          @{ $self->children };
+        if ( $op eq '+' ) {
+            return $left + $right;
+        }
+        elsif ( $op eq '-' ) {
+            return $left - $right;
+        }
+        else {    # *
+            return $left * $right;
+        }
+    }
+}
+
+
+method reduce() {
+    my $class = ref($self);
+
+    if ( $self->is_unit ) {
+        return $self->node;
+    }
+    elsif ( $self->is_number ) {
+        return $self->clone;
     }
     else {
-        return join(
-            ', ',
-            map {
-                my $arg0 = $self->children->[0]->at($_);
-                my $arg1 = $self->children->[1]->at($_);
-                my $format;
-                if ( $self->node eq '*' ) {
-                    if ( $arg0->is_arithmetic ) {
-                        $format = "(%s)%s%s";
-                    }
-                    elsif ( $arg1->is_arithmetic ) {
-                        $format = "%s%s(%s)";
-                    }
+        my ( $left, $right ) = map { $_->reduce; } @{ $self->children };
+        if (   $left->$_isa('Graphics::Grid::Arithmetic')
+            or $right->$_isa('Graphics::Grid::Arithmetic') )
+        {
+            return $class->new(
+                node     => $self->node,
+                children => [ $left, $right ]
+            );
+        }
+        else {
+            if ( $self->node eq '*' ) {    # one of the children be unit
+                my ( $unit, $number ) =
+                  $left->$_isa('Graphics::Grid::Unit')
+                  ? ( $left, $right )
+                  : ( $right, $left );
+                my @value =
+                  map {
+                    $unit->_value_at($_) *
+                      $number->node->[ $_ % @{ $number->node } ]
+                  } ( 0 .. List::AllUtils::max( $unit->elems, $number->elems ) -
+                      1 );
+
+                return Graphics::Grid::Unit->new( \@value, $unit->unit,
+                    $unit->data );
+            }
+            else {
+                if ( $left->is_absolute and $right->is_absolute ) {
+                    my @value = map {
+                        my $a = $left->_transform_absolute_unit_to_cm($_);
+                        my $b = $right->_transform_absolute_unit_to_cm($_);
+                        $self->node eq '+' ? $a + $b : $a - $b;
+                    } (
+                        0 .. List::AllUtils::max( $left->elems, $right->elems )
+                          - 1 );
+                    return Graphics::Grid::Unit->new( \@value, 'cm' );
                 }
-                $format //= "%s%s%s";
-                sprintf( $format,
-                    $arg0->stringify, $self->node, $arg1->stringify );
-            } ( 0 .. $self->elems - 1 )
-        );
+                else {
+                    return $class->new(
+                        node     => $self->node,
+                        children => [ $left, $right ]
+                    );
+                }
+            }
+        }
     }
 }
 
-method _make_operation( $op, $other, $swap = undef ) {
-    return __PACKAGE__->new(
-        node     => $op,
-        children => ( $swap ? [ $other, $self ] : [ $self, $other ] )
-    );
-}
-
-method plus( UnitLike $other, $swap = undef ) {
-    return $self->_make_operation( '+', $other, $swap );
-}
-
-method minus( UnitLike $other, $swap = undef ) {
-    return $self->_make_operation( '-', $other, $swap );
-}
-
-method multiply( ( ArrayRef [Num] | Num ) $other, $swap = undef ) {
-    return $self->_make_operation( '*', $other, $swap );
-}
-
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -167,7 +258,7 @@ Graphics::Grid::UnitArithmetic - Expression created from Graphics::Grid::Unit ob
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -189,7 +280,7 @@ version 0.0001
 
     # or use the function interface
     use Graphics::Grid::Functions qw(:all);
-    my $ua = unit(@params) * 2;
+    my $ua = unit(2, 'inches') * 2;
 
 =head1 DESCRIPTION
 
@@ -203,7 +294,7 @@ value that combines both relative and absolute values.
 Supported operators are C<+>, C<->, and C<*>. A plus or minus
 operation requires both its binary operands are consumers of 
 Graphics::Grid::UnitLike. The multiply operation requires one of
-its operands is consumer of Graphics::Grid::UnitLike, the other
+its operands is consumer of L<Graphics::Grid::UnitLike>, the other
 a number or array ref of numbers.
 
 =head1 ATTRIBUTES
@@ -239,6 +330,14 @@ attribute has the operands.
 
 =head1 METHODS
 
+=head2 elems
+
+Number of effective values in the object.
+
+=head2 length
+
+This is an alias of C<elems()>.
+
 =head2 at($idx)
 
 This method returns an object of the same Graphics::Grid::UnitArithmetic class.
@@ -261,9 +360,28 @@ C<$idx> is applied like wrap-indexing. So below is same as above.
 
     my $ua3 = $ua1->at(4);
 
-=head2 elems
+=head2 slice($indices) 
 
-Return the number of effective elements in the object.
+Slice by indices.
+
+=head2 string()
+
+Returns a string representing the object.
+
+=head2 sum()
+
+Sum the effective unit vector in a unit object.
+
+=head2 append($other)
+
+Append two UnitLike objects. If both are Graphics::Grid::Unit objects,
+this method would return a Graphics::Grid::Unit object. Otherwise,
+it would return a Graphics::Grid::UnitList object.
+
+=head2 insert($other, $after=$self->elems-1)
+
+Insert another UnitLike object after index C<$after>.
+Insert before first element if C<after> is a negative value.
 
 =head2 is_unit
 
@@ -278,9 +396,11 @@ Checks if the object is an array ref of numbers.
 Check is the object is an arithmetic operation. It is equivalent
 to C<!($obj-E<ge>is_unit() or $obj-E<ge>is_number())>.
 
-=head2 stringify
+=head2 reduce()
 
-Stringify the object.
+Try to reduce the tree by evaluating arithmetics.
+Returns a new Graphics::Grid::UnitArithmetic or Graphics::Grid::Unit
+object.
 
 =head1 SEE ALSO
 
@@ -298,7 +418,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
