@@ -4,14 +4,15 @@ package Graphics::Grid::Util;
 
 use Graphics::Grid::Setup;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
-use Exporter 'import';
+use parent qw(Exporter::Tiny);
 
 our @EXPORT_OK = qw(
   dots_to_cm cm_to_dots
   dots_to_inches inches_to_dots
   points_to_cm cm_to_points
+  gcd lcm
 );
 our %EXPORT_TAGS = ( all => \@EXPORT_OK );
 
@@ -24,6 +25,15 @@ fun cm_to_dots( $x, $dpi ) { $x / 2.54 * $dpi; }
 fun points_to_cm($x) { $x / 72.27 * 2.54; }
 fun cm_to_points($x) { $x / 2.54 * 72.27; }
 
+fun gcd( $x, $y ) {
+    ( $x, $y ) = ( $y, $x % $y ) while $y;
+    return $x;
+}
+
+fun lcm( $x, $y ) {
+    return ( $x * $y / gcd( $x, $y ) );
+}
+
 1;
 
 __END__
@@ -38,7 +48,7 @@ Graphics::Grid::Util - Utility functions used internally in Graphics::Grid
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -66,7 +76,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
