@@ -3,62 +3,54 @@ package Graphics::Grid::Viewport;
 # ABSTRACT: Viewport
 
 use Graphics::Grid::Class;
-use MooseX::HasDefaults::RO;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
-use Types::Standard qw(Num Str ArrayRef HashRef);
-use namespace::autoclean;
+use Types::Standard qw(Int InstanceOf Num Str ArrayRef HashRef);
 
 use Graphics::Grid::Types qw(:all);
 use Graphics::Grid::Unit;
 
 
 
+has clip => (is => 'ro', isa => Clip, default => 'inherit' );
+
+
 my $Scale = ( ArrayRef [Num] )->where( sub { @$_ == 2 } );
 
 has [ "xscale", "yscale" ] => (
+    is      => 'ro',
     isa     => $Scale,
     default => sub { [ 0, 1 ] },
 );
 
 
-# TODO
-
-#has clip => (
-#    isa     => Clip,
-#    default => 'inherit',
-#);
-
-
-has angle => ( isa => Num, default => 0 );
-
-#has layout         => ();
-#has layout_pos_row => ();
-#has layout_pos_col => ();
-
-
-has name => (
-    isa     => Str,
-    lazy    => 1,
-    builder => '_build_name',
+has angle => (
+    is      => 'ro',
+    isa     => Num,
+    default => 0
 );
 
-has _uid => (
-    default => sub {
-        state $idx = 0;
-        my $name = "GRID.VP.$idx";
-        $idx++;
-        return $name;
-    },
-    init_arg => undef
+
+has layout => (
+    is  => 'ro',
+    isa => InstanceOf ["Graphics::Grid::Layout"],
+    writer => '_set_layout',
 );
+has [qw(layout_pos_row layout_pos_col)] => (
+    is  => 'ro',
+    isa => ( ArrayRef [Int] )->plus_coercions(ArrayRefFromAny),
+    coerce => 1,
+);
+
+
+
 
 with qw(
+  Graphics::Grid::ViewportLike
   Graphics::Grid::Positional
   Graphics::Grid::Dimensional
-  Graphics::Grid::Justifiable
-  Graphics::Grid::HasGPar
+  Graphics::Grid::HasJust
 );
 
 sub _build_name { $_[0]->_uid; }
@@ -79,7 +71,7 @@ Graphics::Grid::Viewport - Viewport
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -168,7 +160,9 @@ A reader accessor for vertical justification.
 
 =head2 gp
 
-An object of Graphics::Grid::GPar. Default is an empty gpar object.
+An object of L<Graphics::Grid::GPar>. Default is an empty gpar object.
+
+=head2 clip
 
 =head2 xscale
 
@@ -190,6 +184,18 @@ A numeric value indicating the angle of rotation of the viewport. Positive
 values indicate the amount of rotation, in degrees, anticlockwise from the
 positive x-axis. Default is 0.
 
+=head2 layout
+
+A L<Graphics::Grid::Layout> object which splits the viewport into subregions.
+
+=head2 layout_pos_row
+
+Indices of rows occupied by this viewport in its parent's layout.
+
+=head2 layout_pos_col
+
+Indices of columns occupied by this viewport in its parent's layout.
+
 =head2 name
 
 A string to uniquely identify the viewport once it has been pushed onto the
@@ -199,13 +205,15 @@ viewport tree. If not specified, it would be assigned automatically.
 
 L<Graphics::Grid>
 
+L<Graphics::Grid::ViewportLike>
+
 =head1 AUTHOR
 
 Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
