@@ -4,30 +4,39 @@ package Graphics::Grid::ViewportTree;
 
 use Graphics::Grid::Class;
 
-our $VERSION = '0.0001'; # VERSION
+our $VERSION = '0.001'; # VERSION
 
 extends qw(Forest::Tree);
 
-use namespace::autoclean;
-
 use Types::Standard qw(ArrayRef InstanceOf);
 
 use Graphics::Grid::Viewport;
 
+with qw(Graphics::Grid::ViewportLike);
+
 has '+children' => ( isa => ArrayRef [ InstanceOf ['Graphics::Grid::ViewportTree'] ] );
 
-around BUILDARGS => sub {
-    my $orig  = shift;
-    my $class = shift;
+method _build_name() {
+    return $self->_uid('GRID.gTree');
+}
 
-    my %params = @_;
+around BUILDARGS($orig, $class : @rest) {
+    my %params;
+    if (@rest == 1) {
+        %params = %{$rest[0]};
+    }
+    elsif (@rest == 2 and $rest[0]->$_isa('Graphics::Grid::Viewport') ) {
+        %params = ( node => $rest[0], children => $rest[1] );
+    } else {
+        %params = @rest;
+    }
     my $children = ( delete $params{children} ) // [];
     $children =
       [ map { $_->$_isa(__PACKAGE__) ? $_ : __PACKAGE__->new( node => $_ ); }
           @$children ];
 
     $class->$orig( %params, children => $children );
-};
+}
 
 
 # return an arrayref of Viewport from root
@@ -44,7 +53,7 @@ method path_from_root() {
 }
 
 
-method stringify () {
+method string () {
     if ( $self->is_leaf ) {
         return sprintf( "Viewport[%s]", $self->node->name );
     }
@@ -53,7 +62,7 @@ method stringify () {
             "Viewport[%s]->(%s)",
             $self->node->name,
             join( ',',
-                map { $_->stringify() } @{ $self->children } )
+                map { $_->string() } @{ $self->children } )
         );
     }
 }
@@ -74,7 +83,7 @@ Graphics::Grid::ViewportTree - Viewport tree
 
 =head1 VERSION
 
-version 0.0001
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -97,25 +106,27 @@ Get the viewport from the tree node.
 Return an array of viewports, starting from the root node of the whole
 tree, all the way down in the tree and to the calling node.
 
-=head2 stringify()
+=head2 string()
 
-    my $tree_as_a_string = $tree->stringify();
+    my $tree_as_a_string = $tree->string();
 
-Return the tree structure as a string.
+Returns a string to represent the object.
 
 =head1 SEE ALSO
 
-L<Graphics::Grid::Viewport>
-
 L<Forest::Tree>
 
+L<Graphics::Grid::ViewportLike>
+
+L<Graphics::Grid::Viewport>
+
 =head1 AUTHOR
 
 Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Stephan Loyd.
+This software is copyright (c) 2018-2023 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
